; File Name   :	kompaktor-kasette\QS16.rom
; Format      :	Binary file
;------------------------------------------------------------------------------
; Autor: Dr. R. Wobst
; reass: V. Pohlers, 10.06.2013

; Die Aufzeichnung erfolgt in Blcken zu 137 Byte
; 1 Byte Blocknummer + 8 * ( 16 Byte Daten + 1 Byte Prfsumme) = 137 Byte

		cpu	z80

; enum PORTS
CTC0:		 equ 80h			; System CTC0 Kassette,	Beeper
CTC1:		 equ 81h			; System CTC1 Anwenderport
CTC2:		 equ 82h			; System CTC2 Systemuhr
CTC3:		 equ 83h			; System CTC3 Systemuhr
; enum RAM
DMA:		 equ 001Bh			; Zeiger auf Puffer fr Kassetten-E/A
BLNR:		 equ 006Bh			; Blocknummer


;------------------------------------------------------------------------------
; Ende: Stack korrigieren, Interrupt-Routinen wieder auf Stufe 1 setzen
setas:		ex	af, af'
		pop	af
		ex	af, af'

;------------------------------------------------------------------------------
; Q	Umstellen auf Schnellaufzeichnung
;------------------------------------------------------------------------------

quick:		ld	bc, QIKACT
		ld	de, QIKEP
		jr	SETINTV		; Interruptvektor setzen

;------------------------------------------------------------------------------
; N	Zurckstellen auf normale Geschwindigkeit
;------------------------------------------------------------------------------
norm:
		ld	bc, 0FF43h	; IKACT: INTERRUPTROUTINE ZUM SCHREIBEN
		ld	de, 0FFBDh	; IKEP:	INTERRUPTROUTINE ZUM LESEN
;
SETINTV:	ld	(200h),	bc	; Interruptvektor setzen
		ld	(20Ah),	de	; ISR-Tabelle IKEP
		ei
		ret

;------------------------------------------------------------------------------
;Einstellen einer neuen Schnellaufzeichnungsgeschwindigkeit
;QV nn
;nn = 09 .. 99, Default 16
;------------------------------------------------------------------------------
qvalue:		ld	hl, 80h+5   	; Position in Kommandozeile
		ld	a, (hl)		; Zeichen holen
		sub	'0'             ; Ziffer
		ld	b, a
		add	a, a
		add	a, a
		add	a, b
		add	a, a		; *10
		inc	hl		; nchste Stelle addieren
		add	a, (hl)		; nn+30h
		sub	33h ; '3'       ; a = nn-3
qvalue1:	ld	(qkara4+1), a	; nn-3		0-Bit
		ld	(qkaub2+1), a	; nn-3		0-Bit
		ld	b, a
		add	a, a
		add	a, 3		;  a = 2*nn-3
		ld	(qkara2+1), a	; 2*nn-3	1-Bit
		ld	(qkaub3+1), a	; 2*nn-3	1-Bit
		add	a, b		; a= 3*nn-6
		rra			; a= 1.5*nn-3
		sub	3		; a=1.5*nn-6
		ld	(qlstop2+1), a	; 1.5*nn-6	Testwert beim Lesen
		;		
		jr	quick		; Umstellen auf Schnellaufzeichnung

;------------------------------------------------------------------------------
; Interrupt-Routine Stufe 1
; wird bem ersten Eintreffen einer Flanke anstelle IKEP aufgerufen
; wenn der Aufruf aus MAREK CALL LSTOP heraus erfolgte, wird der Ablauf gendert:
; es geht dann mit QMAREK anstelle der orig. OS-Kassetten-Routinen weiter
;------------------------------------------------------------------------------

QIKEP:		push	af
		push	hl
		push	de
		ld	de, 0FF7Bh	; in MAREK hinter LSTOP
		ld	hl, 10
		call	sstk		; Suche	in Stack, an Offset-Pos	HL Wert	DE
					; wenn Wert gefunden, dann Z=1
		pop	de
		pop	hl
;wenn nicht aus MA2: CALL LSTOP heraus gerufen
		jp	nz, 0FF52h	; In IKACT (gleiches Ende wie IKEP)
					; ld (arb),a, pop af, ei, reti
;wenn aus MA2: CALL LSTOP heraus gerufen
		pop	hl		; RET von IKEP
		pop	hl		; RET von LSTOP
		ld	hl, QMAREK	; auf Stack legen
		ex	(sp), hl	; RET von IBYTE
		reti

;------------------------------------------------------------------------------
; Interrupt-Routine  Stufe 1
; wird bem ersten Ausgeben einer Flanke anstelle IKACT aufgerufen
; wenn der Aufruf aus KARAM CALL xxx heraus erfolgte, wird der Ablauf gendert:
; es geht dann mit QKARAM anstelle der orig. OS-Kassetten-Routinen weiter
;------------------------------------------------------------------------------
QIKACT:		push	af
		push	hl
		push	de
		ld	de, 0FEE6h	; in KARAM hinter KARA1
		ld	hl, 12
		call	sstk		; Suche	in Stack, an Offset-Pos	HL Wert	DE
					; wenn Wert gefunden, dann Z=1
		pop	de
		pop	hl
		jp	nz, 0FF51h	; In IKACT
					; xor a, ld (arb),a, pop af, ei, reti
		pop	hl
		pop	hl
		pop	hl
		ld	hl, QKARAM	; Quick	Block schreiben
		ex	(sp), hl
		reti

;------------------------------------------------------------------------------
; Suche	in Stack, an Offset-Pos	HL Wert	DE
; wenn Wert gefunden, dann Z=1
sstk:
		add	hl, sp
		ld	a, e
		cp	(hl)
		ret	nz
		ld	a, d
		inc	hl
		cp	(hl)
		ret

;------------------------------------------------------------------------------
; Quick Block schreiben
; in:	DMA  Blockadresse
; 	BLNR Blocknummer
; 	BC Anzahl der Vortonzeichen
;------------------------------------------------------------------------------

QKARAM:		ex	af, af'         
		push	af
		inc	b
		ld	a, (BLNR)	; BLNR
		cp	b
		ld	a, 3		; CTC Reset
		jr	nz, qkara1
		ld	b, a
qkara1:		out	(CTC1), a
		out	(CTC2), a
		inc	b
		ld	c, b
		ld	hl, QIKACT2	; CTC mit Zeitwert A starten
		ld	(200h),	hl	; neue IKACT
		ld	hl, BLNR
		ld	e, (hl)		; Blocknummer holen
		ld	hl, (DMA)
		ld	d, 8		; 8 * 10h Byte = 128 Byte
		ld	a, 87h		; Steuerbyte CTC
		ex	af, af'
qkara2:		ld	a, 29		; 2*nn-3
		call	QIKACT2		; CTC mit Zeitwert A starten
		; Vorton
qkara3:		halt			; Halbschwingung abwarten
		dec	bc
		inc	b
		djnz	qkara3
		; 2 Trennzeichen (0-Bits)
qkara4:		ld	a, 13		; nn-3
		halt			; Halbschwingung abwarten
		halt			; Halbschwingung abwarten
		halt			; Halbschwingung abwarten
		halt			; Halbschwingung abwarten
		; Blocknummer
		call	QKAUBT		; Blocknummer schreiben 
		; Blcke a 10H Byte + Prfbyte
qkara5:		ld	b, 10h		; 10h Byte
qkara6:		ld	e, (hl)		; Datenbyte holen
		inc	hl
		ld	a, c		; Prfsumme
		add	a, e
		ld	c, a
		call	QKAUBT		; Schreiben eines Bytes
		djnz	qkara6
		; Prfsumme
		ld	e, c
		call	QKAUBT		; Prfsumme schreiben
		; nchster Block
		dec	d		; uerer Schleifenzhler
		jr	nz, qkara5	; 10h Byte
		; Trennzeichen (1-Bit)
		call	qkaub3
		ld	a, 3
		out	(CTC0), a
		or	a
		jp	setas		; Register A' setzen

;------------------------------------------------------------------------------
; Schreiben eines Bytes
; in E: Byte
QKAUBT:		scf			
		jr	qkaub2
qkaub1:		di
		rr	e		; nchstes Bit
		ret	z		; ende, wenn E=, d.h. 8 Bit fertig
		jr	c, qkaub3
; 0-Bit		
qkaub2:		ld	a, 13		; nn-3
		ei
		halt			; 2 Halbschwingungen
		halt
		jr	qkaub1
; 1-Bit		
qkaub3:		ld	a, 29		; 2*nn-3
		or	a
		ei
		halt			; 1 Halbschwingung
		jr	qkaub1

;------------------------------------------------------------------------------
; Interrupt-Routine Stufe 2
; CTC mit Zeitwert A starten
QIKACT2:
		ex	af, af'
		out	(CTC0), a
		ex	af, af'
		out	(CTC0), a
		ei
		reti

;------------------------------------------------------------------------------
; Quick MAREK
; Quick	Block Lesen
; ret: 	BLNR Blocknummer
; 	(PSUM Prfsumme)
; 	CY 0 kein Fehler
; 	   1 Prfsummenfehler
;------------------------------------------------------------------------------
QMAREK:		ld	a, 3		; CTC Reset
		out	(CTC1), a
		out	(CTC2), a
		ld	hl, QIKEP2
		ld	(20Ah),	hl	; neue IKEP
		ld	hl, (DMA)
		ld	c, 0FFh
		ld	d, 8		; 8*10H	Byte = 128 Byte
		; Vorton
qmare1:		ld	b, 10h
qmare2:		call	QLSTOP		; Lesen	eines Zeichens
		jr	nc, qmare1
		djnz	qmare2
qmare3:		ld	b, 2
		; 2 Trennzeichen (0-Bits)
qmare4:		call	QLSTOP		; Lesen	eines Zeichens
		jr	c, qmare3
		djnz	qmare4
		; Blocknummer
		call	QIBYTE		; Lesen	eines Bytes
		ld	a, e
		ld	(BLNR), a
		; Blcke a 10H Byte + Prfbyte
qmare5:		ld	b, 10h
qmare6:		call	QIBYTE		; Lesen	eines Bytes
		ld	(hl), e
		inc	hl
		ld	a, c
		add	a, e
		ld	c, a		; Prfsumme
		djnz	qmare6
		; Prfsumme
		call	QIBYTE		; Lesen	eines Bytes
		ld	a, c
		cp	e		; Vergleichen mit Sollwert
		jr	nz, qmare8
		; nchster Block
		dec	d		; uerer Schleifenzhler
		jr	nz, qmare5
qmare7:		jp	quick
;
qmare8:		scf			; wenn Prfsummenfehler
		jr	qmare7

;------------------------------------------------------------------------------
; Lesen	eines Bytes
; ret E = gelesenes Byte
QIBYTE:
		call	QLSTOP		; Lesen	eines Zeichens
		ld	e, 80h 
qibyte1:	call	QLSTOP		; Lesen	eines Zeichens
		rr	e
		jr	nc, qibyte1
		ret

;------------------------------------------------------------------------------
; Lesen	eines Zeichens
; Ret Cy = gelesenes Bit

;Bit	Wert	0-nn	+Peak 		Cy	
;       nn		1.5*nn-6	vor ccf
;---------------------------------------------
;0	nn-3	-nn+3	0.5nn-3		Cy=1
;1	2*nn-3	-2nn+3	-0.5nn-3	Cy=0


QLSTOP:		xor	a		; A := 0	
		ei
qlstop1:	dec	a		; Interruptroutine setzt Cy
		jr	nc, qlstop1	; Warten auf Flanke
qlstop2:	add	a, 18		; 1.5*nn-6
		ccf
		ret	c		; 1-Bit
		ei
		halt			; bei 0-Bit 2. Flanke abwarten
		or	a		; Cy := 0
		ret

;------------------------------------------------------------------------------
; Interrupt-Routine Stufe 2
QIKEP2:		scf
		reti

